<?php 

use App\Models\Account\Status, App\Models\Account\Account;

class Account_Controller extends Base_Controller {

	public $view = 'account';
	public $uri  = 'account';

	/**
	 * Конструктор
	 */
	public function __construct()
	{
		// Запрещаем гостям для всех страни кроме
		$this->filter('before', 'auth')->except(array('login'));
		
		// Запрещаем авторизованным страницы 
		$this->filter('before', 'guest')->only(array('login'));

		// csrf фильтр
		$this->filter('before', 'csrf')->on('post');

		parent::__construct();
	}

	/**
	 * Method: GET
	 *
	 * Страница профиля
	 */
	public function get_index()
	{
		$user = Account::with(array('status'))
				   ->where(Account::key(), '=', Auth::user()->id)
				   ->first();

		if (!$user)
			return Response::error('404');

		return View::make($this->view .'.profile', array(
			'user' => $user,
		));
	}


	/**
	 * Method: GET
	 *
	 * Cтраница авторизации
	 */
	public function get_login()
	{
		return View::make($this->view .'.login');
	}

	/**
	 * Method: POST
	 *
	 * Авторизация
	 */
	public function post_login()
	{
		$rules = array(
			'name' => 'required',
			'password' => 'required',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$credentials = array(
				'username' => Input::get('name'),
				'password' => Input::get('password')
			);

			if (Auth::attempt($credentials))
				return Redirect::to('/');
			else
				$validator->errors->add('user_login_fail', true);
		}

		// редиректим сохранив данные формы для показа после
		return Redirect::to($this->uri .'/login')->with('errors', $validator->errors)->with_input();
	}

	/**
	 * Method: GET
	 *
	 * Logout
	 */
	public function get_logout()
	{
		Auth::logout();
		
		return Redirect::to($this->uri .'/');
	}
}