<?php

use App\Models\HDL\AnswerBack, App\Models\HDL\Command;

class Hdl_AnswerBack_Controller extends Hdl_Controller {

	public $view = 'hdl.answerback';
	public $uri = 'hdl/answerback';

	/**
	 * Method: GET
	 *
	 * Отрисовываем список ответных команд
	 */
	public function get_index()
	{
		$items = AnswerBack::paginate(Config::get('application.paginate'));

		return View::make($this->view .'.index', array('items' => $items));
	}

	/**
	 * Method: GET
	 *
	 * Страница добавления новой ответной команды
	 */
	public function get_create()
	{
		return View::make($this->view .'.form', array('create' => true, 'commands' => Command::all()));
	}

	/**
	 * Method: POST
	 *
	 * Добавление новой ответной команды
	 */
	public function post_create()
	{
		$rules = array(
			'title'        => 'required|max:50',
			'code'         => 'required|min:3|max:50|match:/^[_a-z]+$/|unique:'. AnswerBack::$table,
			'operate_code' => 'required|size:4|match:/^[0-9a-zA-Z]+$/|unique:'. AnswerBack::$table,
			'commands'     => 'integer_array',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$fields = array(
				'title'          => Input::get('title'),
				'code'           => Input::get('code'),
				'operate_code'   => Input::get('operate_code'),
				'multi_mask'     => Input::get('multi_mask'),
			);

			$new_answerback = AnswerBack::create($fields);

			if ($new_answerback !== false) {
				$commands = Input::get('commands');

				if (!empty($commands))
					$new_answerback->commands()->sync($commands);

				return Redirect::to($this->uri);
			} else
				$validator->errors->add('model_create_fail', true);
		}

		// редиректим сохранив данные формы для показа после
		return Redirect::to($this->uri .'/create')->with('errors', $validator->errors)->with_input();
	}


	/**
	 * Method: GET
	 *
	 * Страница редактирования ответной команды
	 */
	public function get_edit($object_id = null)
	{
		$object_id = (int) $object_id;

		if (!$object_id)
			return Response::error('404');

		$answerback = AnswerBack::find($object_id);

		if (!$answerback)
			return Response::error('404');

		// подготавливаем комманды к удобному виду для работы с формами и не создавая модель АР
		$answerback->commands = $answerback->pivot_relation_array(Command::$answerback_pivot_table, 'command_id');

		return View::make($this->view .'.form', array('answerback' => $answerback, 'create' => false, 'commands' => Command::all()));
	}

	/**
	 * Method: POST
	 *
	 * Редактирование ответной команды
	 */
	public function post_edit()
	{
		$rules = array(
			'id'           => 'required|integer',			
			'title'        => 'required|max:50',
			'code'         => 'required|min:3|max:50|match:/^[_a-z]+$/|unique:'. AnswerBack::$table .',code,'. Input::get('id'),
			'operate_code' => 'required|size:4|match:/^[0-9a-zA-Z]+$/|unique:'. AnswerBack::$table .',operate_code,'. Input::get('id'),
			'commands'     => 'integer_array',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$answerback = AnswerBack::find(Input::get('id'));

			if ($answerback)
			{
				$fields = array(
					'title'          => Input::get('title'),
					'code'           => Input::get('code'),
					'operate_code'   => Input::get('operate_code'),
					'multi_mask'     => Input::get('multi_mask'),
				);

				$answerback->fill($fields);

				if ($answerback->save()) {
					$commands = Input::get('commands');

					if (!empty($commands))
						$answerback->commands()->sync($commands);

					return Redirect::to($this->uri);
				} else
					$validator->errors->add('model_edit_fail', true);
			} else
				$validator->errors->add('model_edit_fail', true);
		}

		// редиректим сохранив данные формы для показа после
		return Redirect::to($this->uri .'/edit/'. Input::get('id'))->with('errors', $validator->errors)->with_input();
	}


	/**
	 * Method: POST
	 *
	 * Удаление ответной команды
	 */
	public function post_delete()
	{
		$rules = array(
			'id' => 'required|integer',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$answerback = AnswerBack::find(Input::get('id'));

			if ($answerback)
			{
				$answerback->params()->delete();
				DB::query('UPDATE '. Command::$table .' SET answerback_id = NULL WHERE answerback_id = ?', array($answerback->id));

				if ($answerback->delete())
					return Redirect::to($this->uri);
				else
					$validator->errors->add('model_delete_fail', true);
			} else
				$validator->errors->add('model_delete_fail', true);
		}

		// редиректим сохранив данные формы для показа после
		return Redirect::to($this->uri .'/edit/'. Input::get('id'))->with('errors', $validator->errors)->with_input();
	}
}