<?php

use App\Models\HDL\AnswerBackParam, App\Models\HDL\AnswerBack, App\Models\HDL\CommandParam;

class Hdl_AnswerBackParam_Controller extends Hdl_Controller {

	public $view = 'hdl.answerbackparam';
	public $uri = 'hdl/answerbackparam';

	/**
	 * Method: GET
	 *
	 * Отрисовываем список параметров ответных команд
	 */
	public function get_index()
	{
		$items = AnswerBackParam::with(array('answerback'))->paginate(Config::get('application.paginate'));

		return View::make($this->view .'.index', array('items' => $items));
	}

	/**
	 * Method: GET
	 *
	 * Страница добавления нового параметра ответной команды
	 */
	public function get_create()
	{
		// добавляем библиотеку для работы с праметрами команды
		Asset::container('footer')->add('answerbackparam_selector', '/js/boogeyman/hdl/selector.js');
		Asset::container('footer')->add('answerbackparam_create', '/js/boogeyman/hdl/answerbackparam/create.js');

		return View::make($this->view .'.form', array('create' => true, 'types' => Config::get('application.answerback.params.types'), 'answerbacks' => AnswerBack::with(array('commands'))->get(), 'commandparams' => CommandParam::all()));
	}

	/**
	 * Method: POST
	 *
	 * Добавление нового параметра ответной команды
	 */
	public function post_create()
	{
		$rules = array(
			'title'         => 'required|max:50',
			'value'         => 'alpha_num',
			'type'          => 'required|integer',
			'offset'        => 'required|integer',
			'size'          => 'required|integer',
			'success_true'  => 'alpha_num',
			'success_false' => 'alpha_num',
			'answerback'    => 'required|integer',
			'commandparams' => 'required',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$answerback = AnswerBack::find(Input::get('answerback'));

			if ($answerback)
			{
				$fields = array(
					'title'           => Input::get('title'),
					'value'           => Input::get('value'),
					'type'            => Input::get('type'),
					'offset'          => Input::get('offset'),
					'size'            => Input::get('size'),
					'success_true'    => Input::get('success_true'),
					'success_false'   => Input::get('success_false'),
					'answerback_id'   => $answerback->id,
					'commandparams'   => Input::get('commandparams'),
				);

				$new_answerback_param = AnswerBackParam::create($fields);

				if ($new_answerback_param !== false)
					return Redirect::to($this->uri);
				else
					$validator->errors->add('model_create_fail', true);
			} else
				$validator->errors->add('model_create_fail', true);
		}

		// редиректим сохранив данные формы для показа после
		return Redirect::to($this->uri .'/create')->with('errors', $validator->errors)->with_input();
	}


	/**
	 * Method: GET
	 *
	 * Страница редактирования нового параметра ответной команды
	 */
	public function get_edit($object_id = null)
	{
		$object_id = (int) $object_id;

		if (!$object_id)
			return Response::error('404');

		$answerback_param = AnswerBackParam::find($object_id);

		if (!$answerback_param)
			return Response::error('404');

		// добавляем библиотеку для работы с праметрами команды
		Asset::container('footer')->add('answerbackparam_selector', '/js/boogeyman/hdl/selector.js');
		Asset::container('footer')->add('answerbackparam_create', '/js/boogeyman/hdl/answerbackparam/create.js');

		return View::make($this->view .'.form', array('answerbackparam' => $answerback_param, 'create' => false, 'types' => Config::get('application.answerback.params.types'), 'answerbacks' => AnswerBack::with(array('commands'))->get(), 'commandparams' => CommandParam::all()));
	}

	/**
	 * Method: POST
	 *
	 * Редактирование параметра команды
	 */
	public function post_edit()
	{
		$rules = array(
			'id'            => 'required|integer',
			'title'         => 'required|max:50',
			'value'         => 'alpha_num',
			'type'          => 'required|integer',
			'offset'        => 'required|integer',
			'size'          => 'required|integer',
			'success_true'  => 'alpha_num',
			'success_false' => 'alpha_num',
			'answerback'    => 'required|integer',
			'commandparams' => 'required',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$answerback = AnswerBack::find(Input::get('answerback'));

			if ($answerback)
			{
				$answerback_param = AnswerBackParam::find(Input::get('id'));

				if ($answerback_param)
				{
					$fields = array(
						'title'           => Input::get('title'),
						'value'           => Input::get('value'),
						'type'            => Input::get('type'),
						'offset'          => Input::get('offset'),
						'size'            => Input::get('size'),
						'success_true'    => Input::get('success_true'),
						'success_false'   => Input::get('success_false'),
						'answerback_id'   => $answerback->id,
						'commandparams'   => Input::get('commandparams'),
					);

					$answerback_param->fill($fields);

					if ($answerback_param->save())
						return Redirect::to($this->uri);
					else
						$validator->errors->add('model_edit_fail', true);
				} else
					$validator->errors->add('model_edit_fail', true);
			} else
				$validator->errors->add('model_edit_fail', true);
		}

		// редиректим сохранив данные формы для показа после
		return Redirect::to($this->uri .'/edit/'. Input::get('id'))->with('errors', $validator->errors)->with_input();
	}


	/**
	 * Method: POST
	 *
	 * Удаление параметра ответной команды
	 */
	public function post_delete()
	{
		$rules = array(
			'id' => 'required|integer',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$answerback_param = AnswerBackParam::find(Input::get('id'));

			if ($answerback_param)
			{
				if ($answerback_param->delete())
					return Redirect::to($this->uri);
				else
					$validator->errors->add('model_delete_fail', true);
			} else
				$validator->errors->add('model_delete_fail', true);
		}

		// редиректим сохранив данные формы для показа после
		return Redirect::to($this->uri .'/edit/'. Input::get('id'))->with('errors', $validator->errors)->with_input();
	}
}