<?php

use App\Models\HDL\Group;

class Hdl_Group_Controller extends Hdl_Controller {

	public $view = 'hdl.group';
	public $uri = 'hdl/group';

	/**
	 * Конструктор
	 */
	public function __construct()
	{
		// Сбрасываем кеш после редактирования элементов
		$this->filter('after', 'single_cache_flush', array('App\Models\HDL\Group'))->on('post');

		parent::__construct();
	}

	/**
	 * Method: GET
	 *
	 * Отрисовываем список групп
	 */
	public function get_index()
	{
		$groups = Group::paginate(Config::get('application.paginate'));

		return View::make($this->view .'.index', array('items' => $groups));
	}

	/**
	 * Method: GET
	 *
	 * Страница добавления новой группы
	 */
	public function get_create()
	{
		return View::make($this->view .'.form', array('create' => true));
	}

	/**
	 * Method: POST
	 *
	 * Добавление новой группы
	 */
	public function post_create()
	{
		$rules = array(
			'title'       => 'required|max:50',
			'weight'      => 'integer',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$fields = array(
				'title'  => Input::get('title'),
				'weight' => Input::get('weight'),
			);

			$new_group = Group::create($fields);

			if($new_group !== false)
				return Redirect::to($this->uri);
			else
				$validator->errors->add('model_create_fail', true);
		}

		// редиректим сохранив данные формы для показа после
		return Redirect::to($this->uri .'/create')->with('errors', $validator->errors)->with_input();
	}

	/**
	 * Method: GET
	 *
	 * Страница редактирования группы
	 */
	public function get_edit($object_id = null)
	{
		$object_id = (int) $object_id;

		if(!$object_id)
			return Response::error('404');

		$group = Group::find($object_id);

		if(!$group)
			return Response::error('404');

		return View::make($this->view .'.form', array('group' => $group, 'create' => false));
	}

	/**
	 * Method: POST
	 *
	 * Редактирование группы
	 */
	public function post_edit()
	{
		$rules = array(
			'id'          => 'required|integer',
			'title'       => 'required|max:50',
			'weight'      => 'integer',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$group = Group::find(Input::get('id'));

			if($group)
			{
				$group->fill(array(
					'title'  => Input::get('title'),
					'weight' => Input::get('weight'),
				));

				if($group->save())
					return Redirect::to($this->uri);
				else
					$validator->errors->add('model_edit_fail', true);
			}else
				$validator->errors->add('model_edit_fail', true);
		}

		// редиректим сохранив данные формы для показа после
		return Redirect::to($this->uri .'/edit/'. Input::get('id'))->with('errors', $validator->errors)->with_input();
	}

	/**
	 * Method: POST
	 *
	 * Удаление группы
	 */
	public function post_delete()
	{
		$rules = array(
			'id' => 'required',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			if (Input::get('id') === 'all') {
				// удаляем все группы
				if (DB::query('DELETE FROM '. Group::$table) !== false)
					return Redirect::to($this->uri);
				else
					$validator->errors->add('model_delete_fail', true);
			} else {
				$group = Group::find(Input::get('id'));

				if ($group)
				{
					if ($group->delete())
						return Redirect::to($this->uri);
					else
						$validator->errors->add('model_delete_fail', true);
				}else
					$validator->errors->add('model_delete_fail', true);
			}
		}

		// редиректим сохранив данные формы для показа после
		return Redirect::to($this->uri .'/edit/'. Input::get('id'))->with('errors', $validator->errors)->with_input();
	}
}