<?php

use App\Models\HDL\DeviceType, App\Models\HDL\ProductDevice;

class Hdl_ProductDevice_Controller extends Hdl_Controller {

	public $view = 'hdl.productdevice';
	public $uri = 'hdl/productdevice';

	/**
	 * Method: GET
	 *
	 * Отрисовываем список продуктов hdl
	 */
	public function get_index()
	{
		$items = ProductDevice::with(array('device_type'))->paginate(500);

		return View::make($this->view .'.index', array('items' => $items));
	}

	/**
	 * Method: GET
	 *
	 * Страница добавления нового продукта hdl
	 */
	public function get_create()
	{
		return View::make($this->view .'.form', array('create' => true, 'device_types' => DeviceType::all()));
	}

	/**
	 * Method: POST
	 *
	 * Добавление нового продукта hdl
	 */
	public function post_create()
	{
		$rules = array(
			'title'       => 'required|max:50',
			'dtype'       => 'required|alpha_num|size:4|unique:'. ProductDevice::$table,
			'descr'       => 'max:200',
			'device_type' => 'required|integer',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$device_type = DeviceType::find(Input::get('device_type'));

			if ($device_type)
			{
				$fields = array(
					'title'          => Input::get('title'),
					'dtype'          => Input::get('dtype'),
					'descr'          => Input::get('descr'),
					'device_type_id' => $device_type->id,
				);

				$new_product_device = ProductDevice::create($fields);

				if ($new_product_device !== false)
					return Redirect::to($this->uri);
				else
					$validator->errors->add('model_create_fail', true);
			} else
				$validator->errors->add('device_type_find_fail', true);
		}

		// редиректим сохранив данные формы для показа после
		return Redirect::to($this->uri .'/create')->with('errors', $validator->errors)->with_input();
	}


	/**
	 * Method: GET
	 *
	 * Страница редактирования продукта hdl
	 */
	public function get_edit($object_id = null)
	{
		$object_id = (int) $object_id;

		if (!$object_id)
			return Response::error('404');

		$product_device = ProductDevice::find($object_id);

		if (!$product_device)
			return Response::error('404');

		return View::make($this->view .'.form', array('product_device' => $product_device, 'create' => false, 'device_types' => DeviceType::all()));
	}

	/**
	 * Method: POST
	 *
	 * Редактирование продукта hdl
	 */
	public function post_edit()
	{
		$rules = array(
			'id'          => 'required|integer',			
			'title'       => 'required|max:50',
			'dtype'       => 'required|alpha_num|size:4|unique:'. ProductDevice::$table .',dtype,'. Input::get('id'),
			'descr'       => 'max:200',
			'device_type' => 'required|integer',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$device_type = DeviceType::find(Input::get('device_type'));

			if ($device_type)
			{
				$product_device = ProductDevice::find(Input::get('id'));

				if ($product_device)
				{
					$fields = array(
						'title'          => Input::get('title'),
						'dtype'          => Input::get('dtype'),
						'descr'          => Input::get('descr'),
						'device_type_id' => $device_type->id,
					);

					$product_device->fill($fields);

					if ($product_device->save())
						return Redirect::to($this->uri);
					else
						$validator->errors->add('model_edit_fail', true);
				} else
					$validator->errors->add('model_edit_fail', true);
			} else
				$validator->errors->add('model_edit_fail', true);
		}

		// редиректим сохранив данные формы для показа после
		return Redirect::to($this->uri .'/edit/'. Input::get('id'))->with('errors', $validator->errors)->with_input();
	}


	/**
	 * Method: POST
	 *
	 * Удаление продукта hdl
	 */
	public function post_delete()
	{
		$rules = array(
			'id' => 'required|integer',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$product_device = ProductDevice::find(Input::get('id'));

			if ($product_device)
			{
				if ($product_device->delete())
					return Redirect::to($this->uri);
				else
					$validator->errors->add('model_delete_fail', true);
			} else
				$validator->errors->add('model_delete_fail', true);
		}

		// редиректим сохранив данные формы для показа после
		return Redirect::to($this->uri .'/edit/'. Input::get('id'))->with('errors', $validator->errors)->with_input();
	}
}