<?php

use App\Models\HDL\Sensor, App\Models\HDL\Device, App\Models\HDL\Command, App\Models\HDL\CommandParam, App\Models\HDL\Group;

class Hdl_Sensor_Controller extends Hdl_Controller {

	public $view = 'hdl.sensor';
	public $uri = 'hdl/sensor';

	/**
	 * Method: GET
	 *
	 * Отрисовываем список сенсоров
	 */
	public function get_index()
	{
		$items = Sensor::with(array('device', 'command'))->paginate(Config::get('application.paginate'));

		return View::make($this->view .'.index', array('items' => $items));
	}

	/**
	 * Method: GET
	 *
	 * Страница добавления нового сенсора
	 */
	public function get_create()
	{
		// добавляем библиотеку для работы с командами устройства и праметрами команды
		Asset::container('footer')->add('switcher_selector', '/js/boogeyman/hdl/selector.js');
		Asset::container('footer')->add('switcher_create', '/js/boogeyman/hdl/switcher/create.js');
		Asset::container('footer')->add('chosen_js', 'js/chosen/chosen.jquery.js');
		Asset::container('footer')->add('chosen_css', 'style/chosen/chosen.css');

		return View::make($this->view .'.form', array('create' => true, 'devices' => Device::all(), 'commands' => Command::all(), 'params' => CommandParam::all(), 'groups' => Group::all()));
	}

	/**
	 * Method: POST
	 *
	 * Добавление нового сенсора
	 */
	public function post_create()
	{
		$rules = array(
			'title'        => 'required|max:50',
			'device'       => 'required|integer',
			'command'      => 'required|integer',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$device  = Device::find(Input::get('device'));
			$command = Command::find(Input::get('command'));

			// пытаемся получить группу
			if (Input::get('group')) {
				$group = Group::find(Input::get('group'));

				// создаем группу если в базе её нету
				if (empty($group))
					$group = Group::create(array('title' => Input::get('group')));
			}

			if ($device && $command)
			{
				$fields = array(
					'title'        => Input::get('title'),
					'device_id'    => $device->id,
					'command_id'   => $command->id,
					'params'       => Input::get('params'),
				);

				if (!empty($group))
					$fields['group_id'] = $group->id;

				$new_sensor = Sensor::create($fields);

				if ($new_sensor !== false)
					return Redirect::to($this->uri);
				else
					$validator->errors->add('model_create_fail', true);
			} else
				$validator->errors->add('model_create_fail', true);
		}

		// редиректим сохранив данные формы для показа после
		return Redirect::to($this->uri .'/create')->with('errors', $validator->errors)->with_input();
	}

	/**
	 * Method: GET
	 *
	 * Страница редактирования переключателя
	 */
	public function get_edit($object_id = null)
	{
		$object_id = (int) $object_id;

		if (!$object_id)
			return Response::error('404');

		$sensor = Sensor::with(array('device', 'command'))->where(Sensor::key(), '=', $object_id)->first();

		if (!$sensor)
			return Response::error('404');

		// добавляем библиотеку для работы с командами устройства и праметрами команды
		Asset::container('footer')->add('switcher_selector', '/js/boogeyman/hdl/selector.js');
		Asset::container('footer')->add('switcher_create', '/js/boogeyman/hdl/switcher/create.js');
		Asset::container('footer')->add('chosen_js', 'js/chosen/chosen.jquery.js');
		Asset::container('footer')->add('chosen_css', 'style/chosen/chosen.css');

		return View::make($this->view .'.form', array('sensor' => $sensor, 'create' => false, 'devices' => Device::all(), 'commands' => Command::all(), 'params' => CommandParam::all(), 'groups' => Group::all()));
	}

	/**
	 * Method: POST
	 *
	 * Редактирование команды
	 */
	public function post_edit()
	{
		$rules = array(
			'id'           => 'required|integer',			
			'title'        => 'required|max:50',
			'device'       => 'required|integer',
			'command'      => 'required|integer',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$sensor    = Sensor::find(Input::get('id'));
			$device    = Device::find(Input::get('device'));
			$command   = Command::find(Input::get('command'));

			// пытаемся получить группу
			if (Input::get('group')) {
				$group = Group::find(Input::get('group'));

				// создаем группу если в базе её нету
				if (empty($group))
					$group = Group::create(array('title' => Input::get('group')));
			}

			if ($device && $command && $sensor)
			{
				$fields = array(
					'title'        => Input::get('title'),
					'device_id'    => $device->id,
					'command_id'   => $command->id,
					'params'       => Input::get('params'),
					'group_id'     => !empty($group) ? $group->id : 0,
				);

				$sensor->fill($fields);

				if ($sensor->save())
					return Redirect::to($this->uri);
				else
					$validator->errors->add('model_edit_fail', true);
			} else
				$validator->errors->add('model_edit_fail', true);
		}

		// редиректим сохранив данные формы для показа после
		return Redirect::to($this->uri .'/edit/'. Input::get('id'))->with('errors', $validator->errors)->with_input();
	}

	/**
	 * Method: POST
	 *
	 * Удаление переключателя
	 */
	public function post_delete()
	{
		$rules = array(
			'id' => 'required',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			if (Input::get('id') === 'all') {
				if (DB::query('DELETE FROM '. Sensor::$table) !== false)
					return Redirect::to($this->uri);
				else
					$validator->errors->add('model_delete_fail', true);
			} else {
				$sensor = Sensor::find(Input::get('id'));

				if ($sensor)
				{
					if ($sensor->delete())
						return Redirect::to($this->uri);
					else
						$validator->errors->add('model_delete_fail', true);
				} else
					$validator->errors->add('model_delete_fail', true);
			}
		}

		// редиректим сохранив данные формы для показа после
		return Redirect::to($this->uri .'/edit/'. Input::get('id'))->with('errors', $validator->errors)->with_input();
	}
}