<?php

use App\Models\HDL\Switcher, App\Models\HDL\Device, App\Models\HDL\Command, App\Models\HDL\CommandParam, App\Models\HDL\Group;

class Hdl_Switcher_Controller extends Hdl_Controller {

	public $view = 'hdl.switcher';
	public $uri = 'hdl/switcher';

	/**
	 * Method: GET
	 *
	 * Отрисовываем список переключателей
	 */
	public function get_index()
	{
		$items = Switcher::with(array('device', 'command'))->paginate(Config::get('application.paginate'));

		return View::make($this->view .'.index', array('items' => $items));
	}

	/**
	 * Method: GET
	 *
	 * Страница добавления нового переключателя
	 */
	public function get_create()
	{
		// добавляем библиотеку для работы с командами устройства и праметрами команды
		Asset::container('footer')->add('switcher_selector', '/js/boogeyman/hdl/selector.js');
		Asset::container('footer')->add('switcher_create', '/js/boogeyman/hdl/switcher/create.js');
		Asset::container('footer')->add('chosen_js', 'js/chosen/chosen.jquery.js');
		Asset::container('footer')->add('chosen_css', 'style/chosen/chosen.css');

		return View::make($this->view .'.form', array('create' => true, 'devices' => Device::all(), 'commands' => Command::all(), 'params' => CommandParam::all(), 'groups' => Group::all()));
	}

	/**
	 * Method: POST
	 *
	 * Добавление нового переключателя
	 */
	public function post_create()
	{
		$rules = array(
			'title'        => 'required|max:50',
			'device'       => 'required|integer',
			'command'      => 'required|integer',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$device  = Device::find(Input::get('device'));
			$command = Command::find(Input::get('command'));

			// пытаемся получить группу
			if (Input::get('group')) {
				$group = Group::find(Input::get('group'));

				// создаем группу если в базе её нету
				if (empty($group))
					$group = Group::create(array('title' => Input::get('group')));
			}

			if ($device && $command)
			{
				$fields = array(
					'title'        => Input::get('title'),
					'device_id'    => $device->id,
					'command_id'   => $command->id,
					'params'       => Input::get('params'),
				);

				if (!empty($group))
					$fields['group_id'] = $group->id;

				$new_switcher = Switcher::create($fields);

				if ($new_switcher !== false)
					return Redirect::to($this->uri);
				else
					$validator->errors->add('model_create_fail', true);
			} else
				$validator->errors->add('model_create_fail', true);
		}

		// редиректим сохранив данные формы для показа после
		return Redirect::to($this->uri .'/create')->with('errors', $validator->errors)->with_input();
	}

	/**
	 * Method: GET
	 *
	 * Страница редактирования переключателя
	 */
	public function get_edit($object_id = null)
	{
		$object_id = (int) $object_id;

		if (!$object_id)
			return Response::error('404');

		$switcher = Switcher::with(array('device', 'command'))->where(Switcher::key(), '=', $object_id)->first();

		if (!$switcher)
			return Response::error('404');

		// добавляем библиотеку для работы с командами устройства и праметрами команды
		Asset::container('footer')->add('switcher_selector', '/js/boogeyman/hdl/selector.js');
		Asset::container('footer')->add('switcher_create', '/js/boogeyman/hdl/switcher/create.js');
		Asset::container('footer')->add('chosen_js', 'js/chosen/chosen.jquery.js');
		Asset::container('footer')->add('chosen_css', 'style/chosen/chosen.css');

		return View::make($this->view .'.form', array('switcher' => $switcher, 'create' => false, 'devices' => Device::all(), 'commands' => Command::all(), 'params' => CommandParam::all(), 'groups' => Group::all()));
	}

	/**
	 * Method: POST
	 *
	 * Редактирование команды
	 */
	public function post_edit()
	{
		$rules = array(
			'id'           => 'required|integer',
			'title'        => 'required|max:50',
			'device'       => 'required|integer',
			'command'      => 'required|integer',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$switcher  = Switcher::find(Input::get('id'));
			$device    = Device::find(Input::get('device'));
			$command   = Command::find(Input::get('command'));

			// пытаемся получить группу
			if (Input::get('group')) {
				$group = Group::find(Input::get('group'));

				// создаем группу если в базе её нету
				if (empty($group))
					$group = Group::create(array('title' => Input::get('group')));
			}

			if ($device && $command && $switcher)
			{
				$fields = array(
					'title'        => Input::get('title'),
					'device_id'    => $device->id,
					'command_id'   => $command->id,
					'params'       => Input::get('params'),
					'group_id'     => !empty($group) ? $group->id : 0,
				);

				$switcher->fill($fields);

				if ($switcher->save())
					return Redirect::to($this->uri);
				else
					$validator->errors->add('model_edit_fail', true);
			} else
				$validator->errors->add('model_edit_fail', true);
		}

		// редиректим сохранив данные формы для показа после
		return Redirect::to($this->uri .'/edit/'. Input::get('id'))->with('errors', $validator->errors)->with_input();
	}

	/**
	 * Method: POST
	 *
	 * Удаление переключателя
	 */
	public function post_delete()
	{
		$rules = array(
			'id' => 'required',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			if (Input::get('id') === 'all') {
				if (DB::query('DELETE FROM '. Switcher::$table) !== false)
					return Redirect::to($this->uri);
				else
					$validator->errors->add('model_delete_fail', true);
			} else {
				$switcher = Switcher::find(Input::get('id'));

				if ($switcher)
				{
					if ($switcher->delete())
						return Redirect::to($this->uri);
					else
						$validator->errors->add('model_delete_fail', true);
				} else
					$validator->errors->add('model_delete_fail', true);
			}
		}

		// редиректим сохранив данные формы для показа после
		return Redirect::to($this->uri .'/edit/'. Input::get('id'))->with('errors', $validator->errors)->with_input();
	}
}