<?php
use App\Models\HDL\Device, App\Models\HDL\DeviceType, App\Models\HDL\Command, App\Models\HDL\CommandParam, App\Models\HDL\Switcher, App\Models\HDL\Sensor, App\Models\HDL\Group;

class Home_Controller extends Base_Controller {
	
	public function __construct()
	{
		// проверяем права
		$this->filter('before', 'access', array('account/login'))->except(array('language'));;

		parent::__construct();
	}

	public function get_index()
	{
		$switchers = Switcher::with(array('group', 'device', 'device.device_type', 'command', 'command.params'))->get();
		$sensors   = Sensor::with(array('group', 'device', 'device.device_type', 'command', 'command.params'))->get();

		return View::make('home.index', array('switchers' => $switchers, 'sensors' => $sensors));
	}

	public function post_index()
	{
		if (Input::get('switch_id'))
		{
			$switcher = Switcher::with(array('device', 'command', 'command.params'))->where(Switcher::key(), '=', Input::get('switch_id'))->first();

			if ($switcher)
				$switcher->toggle(Input::get('switcher_param'));
		}

		if (Input::get('sensor_id'))
		{
			$sensor = Sensor::with(array('device', 'command', 'command.params'))->where(Sensor::key(), '=', Input::get('sensor_id'))->first();

			if ($sensor)
				$sensor->refresh();
		}

		return true;
	}

	/**
	 * Method: GET
	 *
	 * Посылаем ping запрос всем устройствам сети
	 */
	public function get_scan()
	{
		// Получаем команду для сканирования
		$scan_cmd = Command::where('code', '=', 'scan')->first();

		// Получаем бродкаст устройство
		$broadcast_dev = Device::getBroadcastDevice();

		// Отправляем комманду на бродкаст устройство
		$scan_cmd->fire($broadcast_dev);
	}

	/**
	 * Method: GET
	 *
	 * Интерактивная страница добавления новых устройств и сканировния
	 */
	public function get_devices()
	{
		$devices = Device::with(array('device_type'));
		//dd(DB::profile());

		return View::make('home.devices', array('devices' => $devices));
	}

	/**
	 * Method: POST
	 *
	 * Добавление нового устройства аяксом
	 */
	public function post_devices()
	{
		$rules = array(
			'title'       => 'required|max:150',
			'subnet_id'   => 'required|integer',
			'device_id'   => 'required|integer',
			'device_type' => 'required|integer',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$device_type = DeviceType::find(Input::get('device_type'));

			if ($device_type)
			{
				$fields = array(
					'title'          => Input::get('title'),
					'subnet_id'      => Input::get('subnet_id'),
					'device_id'      => Input::get('device_id'),
					'device_type_id' => $device_type->id,
				);
				try {
					$new_device = Device::create($fields);
				} catch (\Exception $e) {
					$validator->errors->add('model_create_fail', true);
				}

				if ($new_device !== false)
					return 'success';
				else
					$validator->errors->add('model_create_fail', true);
			} else
				$validator->errors->add('device_type_find_fail', true);
		}

		return json_encode($validator->errors);
	}

	/**
	 * Method: GET
	 *
	 * Интерактивная форма добавления упраляющих элементов
	 */
	public function get_add_control()
	{
		if (!Input::get('type'))
			return;
		/**
		 * По типу формы определяем устройства, комманды и параметры команд  
		 * Самостоятельно привязываем тип формы к набору типов устройств
		 */
		switch (Input::get('type')) {
			case 'switcher':
				$dtypes = array('rele', 'dimmer', 'universal_switch', 'curtain_switch');
				break;
			case 'sensor':
				$dtypes = array('sensors', 'panels');
				break;
			default:
				return 'wrong type';
		}

		$device_types = DeviceType::with(array('devices', 'commands'))->where_in('code', $dtypes)->get();

		$devices  = array();
		$commands = array();
		foreach ($device_types as $device_type) {
			$devices  = array_merge($devices, $device_type->devices);
			$commands = array_merge($commands, $device_type->commands);
		}

		return View::make('home.control_form', array('create' => true, 'devices' => $devices, 'commands' => $commands, 'params' => CommandParam::where('uf_visible', '=', 1)->get(), 'groups' => Group::all()));
	}

	/**
	 * Method: GET
	 *
	 * Интерактивная форма добавления упраляющих элементов
	 */
	public function post_add_control()
	{
		$rules = array(
			'title'        => 'required|max:50',
			'device'       => 'required|integer',
			'command'      => 'required|integer',
			'type'         => 'required|in:switcher,sensor',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$device  = Device::find(Input::get('device'));
			$command = Command::find(Input::get('command'));

			// пытаемся получить группу
			if (Input::get('group')) {
				$group = Group::find(Input::get('group'));

				// создаем группу если в базе её нету
				if (empty($group))
					$group = Group::create(array('title' => Input::get('group')));
			}

			if ($device && $command)
			{
				$fields = array(
					'title'        => Input::get('title'),
					'device_id'    => $device->id,
					'command_id'   => $command->id,
				);

				if (!empty($group))
					$fields['group_id'] = $group->id;

				$input_params = Input::get('params');
				// если от пользователя какието параметры не пришли мы дополняем их
				foreach ($command->params as $param) {
					if (empty($input_params[$param->id]))
						$input_params[$param->id] = $param->value;
				}
				
				$fields['params'] = $input_params;

				if (Input::get('type') == 'switcher')
					$new_control = Switcher::create($fields);

				if (Input::get('type') == 'sensor')
					$new_control = Sensor::create($fields);

				if ($new_control !== false) {
					// запускаем комманды для определения статуса
					$refreshcommands = Command::with(array('device_type', 'device_type.devices'))->where('periodic', '=', 1)->where('device_type_id', '=', $command->device_type_id)->get();

					// выполняем комманды для определения статуса для конкретного устройства
					foreach ($refreshcommands as $refreshcommand) {
						$refreshcommand->fire($device);
						$refreshcommand->reset_payload();
						// wait some time to send next refresh cmd
						sleep(1);
					}

					// если добавили сенсор то сразу выполняем комманду на его обновление
					if (Input::get('type') == 'sensor')
						$new_control->refresh();

					return 'success';
				}
				else
					$validator->errors->add('model_create_fail', true);
			} else
				$validator->errors->add('model_create_fail', true);
		}

		return json_encode($validator->errors);
	}

	/**
	 * Method: GET
	 *
	 * Смена языка
	 */
	public function get_language()
	{
		$rules = array(
			'lang' => 'required|in:en,ru',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			setcookie('lang', Input::get('lang'));
		}

		return Redirect::to('/');
	}
}