<?php

use Laravel\Messages;

class System_Network_Controller extends System_Controller {

	public $view = 'system.network';
	public $uri = 'system/network';

	/**
	 * Конструктор
	 */
	public function __construct()
	{
		parent::__construct();
	}

	/**
	 * Method: GET
	 *
	 * Показываем настройки сети
	 */
	public function get_index()
	{
		$errors = Session::has('errors') ? Session::get('errors') : new Messages();

		$contents = File::get('/etc/network/interfaces');

		if (!empty($contents))
		{
			$settings = array();

			$matches = array();
			// поиск ip адресса
			preg_match('/address\s+(.+)\s{4}/i', $contents, $matches);
			
			if (!empty($matches[1]))
				$settings['address'] = trim($matches[1]);

			// поиск маски
			preg_match('/netmask\s+(.+)\s{4}/i', $contents, $matches);

			if (!empty($matches[1]))
				$settings['netmask'] = trim($matches[1]);

			// поиск шлюза
			preg_match('/gateway\s+(.+)\s{4}/i', $contents, $matches);
			
			if (!empty($matches[1]))
				$settings['gateway'] = trim($matches[1]);

			if (count($settings) < 3)
				$errors->add('interfaces', 'Неудалось получить все поля из конфигурации');
		} else
			$errors->add('interfaces', 'Файл конфигурации не найден (/etc/network/interfaces)');

		return View::make($this->view .'.index', array('errors' => $errors, 'settings' => $settings));
	}

	/**
	 * Method: GET
	 *
	 * Сохраняем настройки сети
	 */
	public function post_index()
	{
		$rules = array(
			'address'  => 'required|ip:',
			'netmask'  => 'required|ip',
			'gateway'  => 'required|ip',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$contents = File::get('/etc/network/interfaces');

			if (!empty($contents))
			{
				$contents = preg_replace('/(address\s+)(.+)(\s{4})/i', '${1}'. Input::get('address') .'${3}', $contents);
				$contents = preg_replace('/(netmask\s+)(.+)(\s{4})/i', '${1}'. Input::get('netmask') .'${3}', $contents);
				$contents = preg_replace('/(gateway\s+)(.+)(\s{4})/i', '${1}'. Input::get('gateway') .'${3}', $contents);

				if (@File::put('/etc/network/interfaces', $contents))
				{
					$validator->errors->add('success', true);
					return Redirect::to($this->uri .'/index')->with('errors', $validator->errors);
				} else
					$validator->errors->add('interfaces', 'Невозможно изменить файл (/etc/network/interfaces)');
			} else
				$validator->errors->add('interfaces', 'Файл конфигурации не найден (/etc/network/interfaces)');
		}

		// редиректим сохранив данные формы
		return Redirect::to($this->uri .'/index')->with('errors', $validator->errors)->with_input();

	}
}