<?php

use Laravel\Messages, App\Models\System\Update;

class System_Update_Controller extends System_Controller {

	public $view = 'system.update';
	public $uri = 'system/update';

	/**
	 * Конструктор
	 */
	public function __construct()
	{
		parent::__construct();
	}

	/**
	 * Method: GET
	 *
	 * Показываем текущую версию и форму для загрузки обновления
	 */
	public function get_index()
	{
		$errors = Session::has('errors') ? Session::get('errors') : new Messages();

		$current_rev = File::get(Config::get('application.updater.rev_file'));

		$viewdata = array();

		if (!empty($current_rev))
			$viewdata['revision'] = $current_rev;
		else
			$errors->add('head_err', 'Не удалось считать текущую версию');

		return View::make($this->view .'.index', array('errors' => $errors, 'viewdata' => $viewdata));
	}

	/**
	 * Method: GET
	 *
	 * Добавляем задачу на обновление и перезапускаем систему
	 */
	public function post_index()
	{
		$rules = array(
			'archive' => 'required|mimes:zip',
		);

		$validator = Validator::make(Input::all(), $rules);

		if ($validator->valid())
		{
			$filepath = Config::get('application.updater.archive_path');

			// проверяем существование пути, если пути нет то пытаемся создать
			if (File::mkdir($filepath))
			{
				$filename = Input::file('archive.name');

				// перемещаем файл в хранилище
				if (Input::upload('archive', $filepath , $filename))
				{
					$upd_fields = array(
						'archive' => $filepath . $filename,
						'status'  => Config::get('application.updater.status.new'),
					);

					// добавляем задачу на обновление
					if (Update::create($upd_fields))
					{
						// перезапускаем систему
						system("sudo /sbin/reboot");

						return View::make($this->view .'.reboot');
					} else
						$validator->errors->add('head_err', 'Ошибка добавления задачи на обновление.');
				} else
					$validator->errors->add('head_err', 'Ошибка перемещения файла в хранилище.');
			} else
				$validator->errors->add('head_err', 'Ошибка генерации места сохранения файла.');
		}

		// редиректим сохранив данные формы
		return Redirect::to($this->uri .'/index')->with('errors', $validator->errors)->with_input();
	}
}