<?php namespace App\Models\HDL;

use App\Models\HDL\BaseCommand, App\Models\HDL\Command, \Config, \Cache;

class AnswerBack extends BaseCommand {
	public static $table = 'hdl_answerback';

	public static $connection = 'mysql_read';

	public function params()
	{
		return $this->has_many('App\Models\HDL\AnswerBackParam', 'answerback_id');
	}

	public function commands()
	{
		return $this->has_many_and_belongs_to('App\Models\HDL\Command', Command::$answerback_pivot_table);
	}

	/**
	 * Custom field for command status
	 */
	public function set_success($input)
	{
		$this->success = $input;
	}

	/**
	 * Custom field for command status
	 */
	public function get_success()
	{
		return $this->success;
	}

	/**
	 * Parsing input data from hdl packet and filling params
	 *
	 * @return AnswerBack filled with params | false on error
	 */
	public function parse($packet='')
	{
		$packet = !empty($packet) ? $packet : $this->packet;

		if (empty($packet))
			return false;

		if (!is_string($packet))
			return false;

		// присваиваем пакет для объекта
		$this->packet = $packet;

		// проходим по всем параметрам и заполняем
		foreach ($this->params as $param) {
			// вырезаем значение из пакета
			$param->value = substr($this->packet, $param->offset, $param->size);

			// если параметр описывает статус выполнения команды то помечаем команду
			if ($param->type == Config::get('application.answerback.params.types.success'))
				$this->success = ($param->value == $param->success_true);

			// приводим значение к десятичному виду
			$param->value = hexdec($param->value);
		}
	}

	/**
	 * Checks is control item can be updated by current answerback
	 *
	 * @param item - control item obj
	 * @param external_params - external params set
	 * @return boolean
	 */
	public function is_valid($item, $external_params = null)
	{
		$params = !empty($this->params) ? $this->params : $external_params;

		if (empty($params))
			return false;

		$item_params = $item->params;

		if (empty($item_params))
			return false;

		foreach ($params as $param)
			// проверяем параметры фиксированого типа
			if ($param->type == Config::get('application.answerback.params.types.fix'))
				foreach ($param->commandparams as $cmdparamid)
					if (isset($item_params[$cmdparamid]))
						if ($item_params[$cmdparamid] != $param->value)
							return false;

		return true;
	}

	/**
	 * Updating control item with current answerback params
	 *
	 * @param item - control item obj
	 * @param external_params - external params set
	 * @return boolean
	 */
	public function refresh($item, $external_params = null)
	{
		$this_params = !empty($external_params) ? $external_params : $this->params;

		if (empty($this_params))
			return false;

		// если команда выполнилась безуспешно
		if (!empty($this->success))
			if ($this->success === false)
				return false;

		$item_params = $item->params;

		if (empty($item_params))
			return false;

		foreach ($this_params as $param) {
			// работаем только с параметрами обычного типа
			if ($param->type == Config::get('application.answerback.params.types.common')) {
				foreach ($param->commandparams as $cmdparamid) {
					if (isset($item_params[$cmdparamid])) {
						$item_params[$cmdparamid] = $param->value;
					}
				}
			}
		}

		$item->params = $item_params;

		try {
			// обновляем объект в базе
			//$result = $item->save();

			// вариант с сохранением параметров в кеше но без обновления в базе
			Cache::put($item::$table .'_'. $item->id, $item->get_attribute('params'), 0);

			$result = true;
		} catch (\Exception $e) {
			// если почемуто не удалось обновится пишем ошибку и продолжаем работать дальше
			echo '	[error] ', $e->getMessage(); ob_flush();
			$result = false;
		}

		return $result;
	}
}