<?php namespace App\Models\HDL;

use App\Models\EloquentFix, Config, Input, App\Models\HDL\Device;

class BaseCommand extends EloquentFix {
	/**
	 * crc table
	 */
	static $crc_tab = array(
		0x0000, 0x1021, 0x2042, 0x3063, 0x4084, 0x50a5, 0x60c6, 0x70e7,
		0x8108, 0x9129, 0xa14a, 0xb16b, 0xc18c, 0xd1ad, 0xe1ce, 0xf1ef,
		0x1231, 0x0210, 0x3273, 0x2252, 0x52b5, 0x4294, 0x72f7, 0x62d6,
		0x9339, 0x8318, 0xb37b, 0xa35a, 0xd3bd, 0xc39c, 0xf3ff, 0xe3de,
		0x2462, 0x3443, 0x0420, 0x1401, 0x64e6, 0x74c7, 0x44a4, 0x5485,
		0xa56a, 0xb54b, 0x8528, 0x9509, 0xe5ee, 0xf5cf, 0xc5ac, 0xd58d,
		0x3653, 0x2672, 0x1611, 0x0630, 0x76d7, 0x66f6, 0x5695, 0x46b4,
		0xb75b, 0xa77a, 0x9719, 0x8738, 0xf7df, 0xe7fe, 0xd79d, 0xc7bc,
		0x48c4, 0x58e5, 0x6886, 0x78a7, 0x0840, 0x1861, 0x2802, 0x3823,
		0xc9cc, 0xd9ed, 0xe98e, 0xf9af, 0x8948, 0x9969, 0xa90a, 0xb92b,
		0x5af5, 0x4ad4, 0x7ab7, 0x6a96, 0x1a71, 0x0a50, 0x3a33, 0x2a12,
		0xdbfd, 0xcbdc, 0xfbbf, 0xeb9e, 0x9b79, 0x8b58, 0xbb3b, 0xab1a,
		0x6ca6, 0x7c87, 0x4ce4, 0x5cc5, 0x2c22, 0x3c03, 0x0c60, 0x1c41,
		0xedae, 0xfd8f, 0xcdec, 0xddcd, 0xad2a, 0xbd0b, 0x8d68, 0x9d49,
		0x7e97, 0x6eb6, 0x5ed5, 0x4ef4, 0x3e13, 0x2e32, 0x1e51, 0x0e70,
		0xff9f, 0xefbe, 0xdfdd, 0xcffc, 0xbf1b, 0xaf3a, 0x9f59, 0x8f78,
		0x9188, 0x81a9, 0xb1ca, 0xa1eb, 0xd10c, 0xc12d, 0xf14e, 0xe16f,
		0x1080, 0x00a1, 0x30c2, 0x20e3, 0x5004, 0x4025, 0x7046, 0x6067,
		0x83b9, 0x9398, 0xa3fb, 0xb3da, 0xc33d, 0xd31c, 0xe37f, 0xf35e,
		0x02b1, 0x1290, 0x22f3, 0x32d2, 0x4235, 0x5214, 0x6277, 0x7256,
		0xb5ea, 0xa5cb, 0x95a8, 0x8589, 0xf56e, 0xe54f, 0xd52c, 0xc50d,
		0x34e2, 0x24c3, 0x14a0, 0x0481, 0x7466, 0x6447, 0x5424, 0x4405,
		0xa7db, 0xb7fa, 0x8799, 0x97b8, 0xe75f, 0xf77e, 0xc71d, 0xd73c,
		0x26d3, 0x36f2, 0x0691, 0x16b0, 0x6657, 0x7676, 0x4615, 0x5634,
		0xd94c, 0xc96d, 0xf90e, 0xe92f, 0x99c8, 0x89e9, 0xb98a, 0xa9ab,
		0x5844, 0x4865, 0x7806, 0x6827, 0x18c0, 0x08e1, 0x3882, 0x28a3,
		0xcb7d, 0xdb5c, 0xeb3f, 0xfb1e, 0x8bf9, 0x9bd8, 0xabbb, 0xbb9a,
		0x4a75, 0x5a54, 0x6a37, 0x7a16, 0x0af1, 0x1ad0, 0x2ab3, 0x3a92,
		0xfd2e, 0xed0f, 0xdd6c, 0xcd4d, 0xbdaa, 0xad8b, 0x9de8, 0x8dc9,
		0x7c26, 0x6c07, 0x5c64, 0x4c45, 0x3ca2, 0x2c83, 0x1ce0, 0x0cc1,
		0xef1f, 0xff3e, 0xcf5d, 0xdf7c, 0xaf9b, 0xbfba, 0x8fd9, 0x9ff8,
		0x6e17, 0x7e36, 0x4e55, 0x5e74, 0x2e93, 0x3eb2, 0x0ed1, 0x1ef0
	);

	/**
	 * initial hdl command packet
	 */
	static $init_payload = array(0xC0, 0xA8, 0x0A, 0x64, 0x48, 0x44, 0x4C, 0x4D, 0x49, 0x52, 0x41, 0x43, 0x4C, 0x45, 0xAA, 0xAA);
	// initial payload for Smarthome G4
	//static $init_payload = array(0xC0, 0xA8, 0x0A, 0xC8, 0x53, 0x4D, 0x41, 0x52, 0x54, 0x43, 0x4C, 0x4F, 0x55, 0x44, 0xAA, 0xAA);

	/**
	 * Constructor
	 *
	 * @return parent constructor
	 */
	public function __construct($attributes = array(), $exists = false)
	{
		//setting hdl headers
		$this->payload = self::$init_payload;

		return parent::__construct($attributes, $exists);
	}

	/**
	 * Custom field for hdl command
	 */
	public function set_payload($payload)
	{
		$this->payload = $payload;
	}

	/**
	 * Custom field for hdl command
	 */
	public function get_payload()
	{
		return $this->payload;
	}

	/**
	 * Custom field for input packet
	 */
	public function set_packet($packet)
	{
		$this->packet = $packet;
	}

	/**
	 * Custom field for input packet
	 */
	public function get_packet()
	{
		return $this->packet;
	}

	/**
	 * Custom field for input parsed packet
	 */
	public function set_parsed_packet($packet)
	{
		$this->parsed_packet = $packet;
	}

	/**
	 * Custom field for input parsed packet
	 */
	public function get_parsed_packet()
	{
		return $this->parsed_packet;
	}

	/**
	 * Adding crc in last two bytes of packet.
	 *
	 * @return bool
	 */
	public static function pack_crc(&$packet)
	{
		$crc = 0;
		$mask = 255; // mask 8 bit 

		for ($i = 16; $i < count($packet); $i++) {

			$dat = ($crc >> 8) & $mask;

			$crc <<= 8;

			$crc ^= self::$crc_tab[$dat ^ $packet[$i]];
		}

		$packet[] = (($crc >> 8) & $mask);
		$packet[] = $crc & $mask; // get last 8 bit

		return true;
	}

	/**
	 * Set command payload to initial state
	 */
	public function reset_payload()
	{
		return $this->payload = self::$init_payload;
	}

	/**
	 * Checking crc in last two bytes of packet.
	 *
	 * @return bool
	 */
	public static function check_crc(&$packet)
	{
		$crc = 0;
		$mask = 255; // mask 8 bit 

		for ($i = 16; $i < count($packet) - 2; $i++) { 
			$dat = ($crc >> 8) & $mask;

			$crc <<= 8;

			$crc ^= self::$crc_tab[$dat ^ $packet[$i]];
		}

		$first_byte = (($crc >> 8) & $mask);

		$second_byte = $crc & $mask; // get last 8 bit

		if (($packet[count($packet)-2] == $first_byte) && ($packet[count($packet)-1] == $second_byte))
			return true;

		return false;
	}

	/**
	 * Executing command with parameters
	 *
	 * @return bool
	 */
	public function fire($device, $param_vals = null)
	{
		$headers_len = count($this->payload);

		$this->payload[] = 9 + 2 + count($this->params); // data length
		$this->payload[] = Config::get('application.command.sender.subnet_id'); // sender subnet id
		$this->payload[] = Config::get('application.command.sender.device_id'); // sender device id
		$this->payload[] = Config::get('application.command.sender.device_type_high'); // original device type: higher then 8
		$this->payload[] = Config::get('application.command.sender.device_type_low'); // original device type: lower then 8
		$this->payload[] = hexdec(substr($this->operate_code, 0, 2)); // operate code: higher then 8
		$this->payload[] = hexdec(substr($this->operate_code, 2, 4));; // operate code: lower then 8
		$this->payload[] = (int) $device->subnet_id; // subnet ID of targeted device
		$this->payload[] = (int) $device->device_id; // device ID of targeted device

		if (empty($this->payload_body)) {
			foreach ($this->params as $param) {

				// пропускаем мнимые параметры, они предназначены для хранения данных у сенсоров
				if ($param->type == Config::get('application.command.params.types.noop')) {
					// уменьшаем размер пакета
					$this->payload[$headers_len]--;

					continue;
				}
				
				$this->payload[] = (!empty($param_vals[$param->id]) || $param_vals[$param->id] === '0') ? (int) $param_vals[$param->id] : (int) $param->value;
			}
		} else {
			$this->payload = $this->payload + str_split($this->payload_body, 2);
		}

		self::pack_crc($this->payload);

		// отладочная печать
		//$this->dump();

		$broadcast_string = '';
		foreach ($this->payload as $byte)
			$broadcast_string .= chr($byte);

		$sock = socket_create(AF_INET, SOCK_DGRAM, SOL_UDP);
		socket_set_option($sock, SOL_SOCKET, SO_BROADCAST, 1);
		socket_sendto($sock, $broadcast_string, strlen($broadcast_string), 0, Config::get('application.command.adress'), Config::get('application.command.port'));
	}

	/**
	 * Debug print
	 *
	 * @return null
	 */
	public function dump()
	{
		foreach ($this->payload as $byte)
			echo $byte, ' - ', chr($byte), ' - ', dechex($byte), chr(10);
		dd(1);
	}

	/**
	 * Parsing incoming hdl packet
	 * 
	 * @return array | bool
	 */
	public static function parsePacket($packet)
	{
		if (empty($packet))
			return false;

		if (!is_string($packet))
			return false;

		$packet_len = strlen($packet);

		$packet_array = array(
			'head'  => substr($packet, 0, 32),
			'len'   => substr($packet, 32, 2),
			's_sub' => substr($packet, 34, 2),
			's_dev' => substr($packet, 36, 2),
			'dtype' => substr($packet, 38, 4),
			'code'  => substr($packet, 42, 4),
			't_sub' => substr($packet, 46, 2),
			't_dev' => substr($packet, 48, 2),
			'data'  => substr($packet, 50, $packet_len - 54),
			'crc'   => substr($packet, $packet_len - 4, 4),
		);

		return $packet_array;
	}

	/**
	 * Getting sender device from parsed packet
	 *
	 * @return HDL\Device | bool
	 */
	public function getSenderDevice()
	{
		if (empty($this->parsed_packet))
			return false;

		if (empty($this->parsed_packet['s_sub']) || empty($this->parsed_packet['s_dev']))
			return false;

		return Device::where('subnet_id', '=', hexdec($this->parsed_packet['s_sub']))->where('device_id', '=', hexdec($this->parsed_packet['s_dev']))->first();
	}
}