<?php

use App\Models\System\Update, \Config, \File;

class Updater_Task {

	public function run($arguments)
	{
		echo chr(10),'Updater task.',
			 chr(10), chr(10), 'Usage: ', chr(10), '	php artisan updater:update', chr(10);
	}

	/**
	 * Метод занимается обновлением системы если есть соотвествущие задачи
	 * 
	 * Метод вызывается при старте системы в rc скрипте
	 */
	public function update()
	{
		// получаем последнюю задачу на обновление
		$task = Update::where('status', '=', Config::get('application.updater.status.new'))
				->order_by('created_at', 'desc')
				->first();

		if ($task)
		{
			echo '	', 'task found - ', $task->archive;

			// помечаем другие не выполненые задачи на обновление как пропущено, если такие есть
			DB::table(Update::$table)
			->where('status', '=', Config::get('application.updater.status.new'))
			->where('id', '!=', $task->id)
			->update(array('status' => Config::get('application.updater.status.skip'), 'message' => 'skiped'));

			// проверяем существование файла из задания
			if (!File::exists($task->archive))
			{
				$task->fill(array(
					'status' => Config::get('application.updater.status.error'),
					'message' => 'update file does not exist',
				))->save();
				exit;
			}

			// проверяем что файл архив
			if (!File::is('zip', $task->archive))
			{
				$task->fill(array(
					'status' => Config::get('application.updater.status.error'),
					'message' => 'update file is not zip archive',
				))->save();
				exit;
			}

			// распаковываем архив в корень приложенния с заменой файлов
			$zip = new ZipArchive;

			if (!$zip->open($task->archive) === TRUE)
			{
				$task->fill(array(
					'status' => Config::get('application.updater.status.error'),
					'message' => 'failed to open zip archive',
				))->save();
				exit;
			}

			if (!$zip->extractTo(path('base')))
			{
				$task->fill(array(
					'status' => Config::get('application.updater.status.error'),
					'message' => 'failed to extract zip archive',
				))->save();
				exit;
			}

			$zip->close();

			// выполняем связанные скрипты
			$shell_ret = shell_exec('./updater.sh');

			$task->fill(array(
				'status' => Config::get('application.updater.status.good'),
				'message' => 'update successfully completed: '. substr($shell_ret, 0, 4000),
			))->save();
		}

		echo '	', 'updater done';
	}
}